@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2004-2008 David Heinemeier Hansson
+# Copyright (c) 2004-2014 David Heinemeier Hansson
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -21,32 +21,38 @@
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #++
 
-begin
-  require 'action_controller'
-rescue LoadError
-  actionpack_path = "#{File.dirname(__FILE__)}/../../actionpack/lib"
-  if File.directory?(actionpack_path)
-    $:.unshift actionpack_path
-    require 'action_controller'
+require 'abstract_controller'
+require 'action_mailer/version'
+
+# Common Active Support usage in Action Mailer
+require 'active_support/rails'
+require 'active_support/core_ext/class'
+require 'active_support/core_ext/module/attr_internal'
+require 'active_support/core_ext/string/inflections'
+require 'active_support/lazy_load_hooks'
+
+module ActionMailer
+  extend ::ActiveSupport::Autoload
+
+  eager_autoload do
+    autoload :Collector
   end
+
+  autoload :Base
+  autoload :DeliveryMethods
+  autoload :InlinePreviewInterceptor
+  autoload :MailHelper
+  autoload :Preview
+  autoload :Previews, 'action_mailer/preview'
+  autoload :TestCase
+  autoload :TestHelper
+  autoload :MessageDelivery
+  autoload :DeliveryJob
 end
 
-require 'action_mailer/vendor'
-require 'tmail'
+autoload :Mime, 'action_dispatch/http/mime_type'
 
-require 'action_mailer/base'
-require 'action_mailer/helpers'
-require 'action_mailer/mail_helper'
-require 'action_mailer/quoting'
-require 'action_mailer/test_helper'
-
-require 'net/smtp'
-
-ActionMailer::Base.class_eval do
-  include ActionMailer::Quoting
-  include ActionMailer::Helpers
-
-  helper MailHelper
+ActiveSupport.on_load(:action_view) do
+  ActionView::Base.default_formats ||= Mime::SET.symbols
+  ActionView::Template::Types.delegate_to Mime
 end
-
-silence_warnings { TMail::Encoder.const_set("MAX_LINE_LEN", 200) }
