@@ -33,7 +33,7 @@ module ActionMailer
 
       add_delivery_method :sendmail, Mail::Sendmail,
         location:  '/usr/sbin/sendmail',
-        arguments: '-i -t'
+        arguments: '-i'
 
       add_delivery_method :test, Mail::TestMailer
     end
