@@ -3,6 +3,23 @@ require 'abstract_unit'
 class TestTestMailer < ActionMailer::Base
 end
 
+class MailerDeliveriesClearingTest < ActionMailer::TestCase
+  def before_setup
+    ActionMailer::Base.deliveries << 'better clear me, setup'
+    super
+  end
+
+  def after_teardown
+    super
+    assert_equal [], ActionMailer::Base.deliveries
+  end
+
+  def test_deliveries_are_cleared_on_setup_and_teardown
+    assert_equal [], ActionMailer::Base.deliveries
+    ActionMailer::Base.deliveries << 'better clear me, teardown'
+  end
+end
+
 class CrazyNameMailerTest < ActionMailer::TestCase
   tests TestTestMailer
 
