@@ -6,7 +6,7 @@ module ActionController
     end
 
     def model_name_from_record_or_class(record_or_class)
-      (record_or_class.is_a?(Class) ? record_or_class : convert_to_model(record_or_class).class).model_name
+      convert_to_model(record_or_class).model_name
     end
   end
 end
