@@ -9,6 +9,7 @@ end
 
 class SendFileController < ActionController::Base
   include TestFileUtils
+  include ActionController::Testing
   layout "layouts/standard" # to make sure layouts don't interfere
 
   attr_writer :options
@@ -30,11 +31,8 @@ class SendFileWithActionControllerLive < SendFileController
 end
 
 class SendFileTest < ActionController::TestCase
-  tests SendFileController
   include TestFileUtils
 
-  Mime::Type.register "image/png", :png unless defined? Mime::PNG
-
   def setup
     @controller = SendFileController.new
     @request = ActionController::TestRequest.new
@@ -201,8 +199,6 @@ class SendFileTest < ActionController::TestCase
     end
   end
 
-  tests SendFileWithActionControllerLive
-
   def test_send_file_with_action_controller_live
     @controller = SendFileWithActionControllerLive.new
     @controller.options = { :content_type => "application/x-ruby" }
