@@ -179,9 +179,9 @@ module ActionView
           if value.is_a?(Array)
             value = escape ? safe_join(value, " ") : value.join(" ")
           else
-            value = escape ? ERB::Util.unwrapped_html_escape(value) : value
+            value = escape ? ERB::Util.unwrapped_html_escape(value) : value.to_s
           end
-          %(#{key}="#{value}")
+          %(#{key}="#{value.gsub('"'.freeze, '&quot;'.freeze)}")
         end
     end
   end
