@@ -1,6 +1,33 @@
 require 'thread_safe'
 
 module ActionView
+  class PartialIteration
+    # The number of iterations that will be done by the partial.
+    attr_reader :size
+
+    # The current iteration of the partial.
+    attr_reader :index
+
+    def initialize(size)
+      @size  = size
+      @index = 0
+    end
+
+    # Check if this is the first iteration of the partial.
+    def first?
+      index == 0
+    end
+
+    # Check if this is the last iteration of the partial.
+    def last?
+      index == size - 1
+    end
+
+    def iterate! # :nodoc:
+      @index += 1
+    end
+  end
+
   # = Action View Partials
   #
   # There's also a convenience method for rendering sub templates within the current controller that depends on a
@@ -56,8 +83,12 @@ module ActionView
   #   <%= render partial: "ad", collection: @advertisements %>
   #
   # This will render "advertiser/_ad.html.erb" and pass the local variable +ad+ to the template for display. An
-  # iteration counter will automatically be made available to the template with a name of the form
-  # +partial_name_counter+. In the case of the example above, the template would be fed +ad_counter+.
+  # iteration object will automatically be made available to the template with a name of the form
+  # +partial_name_iteration+. The iteration object has knowledge about which index the current object has in
+  # the collection and the total size of the collection. The iteration object also has two convenience methods,
+  # +first?+ and +last?+. In the case of the example above, the template would be fed +ad_iteration+.
+  # For backwards compatibility the +partial_name_counter+ is still present and is mapped to the iteration's
+  # +index+ method.
   #
   # The <tt>:as</tt> option may be used when rendering partials.
   #
@@ -281,6 +312,8 @@ module ActionView
       end
     end
 
+    private
+
     def render_collection
       return nil if @collection.blank?
 
@@ -300,7 +333,7 @@ module ActionView
         layout = find_template(layout.to_s, @template_keys)
       end
 
-      object ||= locals[as]
+      object = locals[as] if object.nil? # Respect object when object is false
       locals[as] = object
 
       content = @template.render(view, locals) do |*name|
@@ -322,25 +355,29 @@ module ActionView
     # respond to +to_partial_path+ in order to setup the path.
     def setup(context, options, block)
       @view   = context
-      partial = options[:partial]
-
       @options = options
-      @locals  = options[:locals] || {}
       @block   = block
+
+      @locals  = options[:locals] || {}
       @details = extract_details(options)
 
       prepend_formats(options[:formats])
 
-      if String === partial
-        @object     = options[:object] if options.has_key?(:object)
-        @path       = partial
-        @collection = collection
-      else
-        @object = partial
+      partial = options[:partial]
 
-        if @collection = collection_from_object || collection
+      if String === partial
+        @has_object = options.key?(:object)
+        @object     = options[:object]
+        @collection = collection_from_options
+        @path       = partial
+      else
+        @has_object = true
+        @object = partial
+        @collection = collection_from_object || collection_from_options
+
+        if @collection
           paths = @collection_data = @collection.map { |o| partial_path(o) }
-          @path = paths.uniq.size == 1 ? paths.first : nil
+          @path = paths.uniq.one? ? paths.first : nil
         else
           @path = partial_path
         end
@@ -352,7 +389,7 @@ module ActionView
       end
 
       if @path
-        @variable, @variable_counter = retrieve_variable(@path, as)
+        @variable, @variable_counter, @variable_iteration = retrieve_variable(@path, as)
         @template_keys = retrieve_template_keys
       else
         paths.map! { |path| retrieve_variable(path, as).unshift(path) }
@@ -361,7 +398,7 @@ module ActionView
       self
     end
 
-    def collection
+    def collection_from_options
       if @options.key?(:collection)
         collection = @options[:collection]
         collection.respond_to?(:to_ary) ? collection.to_ary : []
@@ -373,9 +410,7 @@ module ActionView
     end
 
     def find_partial
-      if path = @path
-        find_template(path, @template_keys)
-      end
+      find_template(@path, @template_keys) if @path
     end
 
     def find_template(path, locals)
@@ -385,19 +420,22 @@ module ActionView
 
     def collection_with_template
       view, locals, template = @view, @locals, @template
-      as, counter = @variable, @variable_counter
+      as, counter, iteration = @variable, @variable_counter, @variable_iteration
 
       if layout = @options[:layout]
         layout = find_template(layout, @template_keys)
       end
 
-      index = -1
+      partial_iteration = PartialIteration.new(@collection.size)
+      locals[iteration] = partial_iteration
+
       @collection.map do |object|
-        locals[as]      = object
-        locals[counter] = (index += 1)
+        locals[as]        = object
+        locals[counter]   = partial_iteration.index
 
         content = template.render(view, locals)
         content = layout.render(view, locals) { content } if layout
+        partial_iteration.iterate!
         content
       end
     end
@@ -407,16 +445,20 @@ module ActionView
       cache = {}
       keys  = @locals.keys
 
-      index = -1
-      @collection.map do |object|
-        index += 1
-        path, as, counter = collection_data[index]
+      partial_iteration = PartialIteration.new(@collection.size)
 
-        locals[as]      = object
-        locals[counter] = index
+      @collection.map do |object|
+        index = partial_iteration.index
+        path, as, counter, iteration = collection_data[index]
+
+        locals[as]        = object
+        locals[counter]   = index
+        locals[iteration] = partial_iteration
 
         template = (cache[path] ||= find_template(path, keys + [as, counter]))
-        template.render(view, locals)
+        content = template.render(view, locals)
+        partial_iteration.iterate!
+        content
       end
     end
 
@@ -466,8 +508,11 @@ module ActionView
 
     def retrieve_template_keys
       keys = @locals.keys
-      keys << @variable if defined?(@object) || @collection
-      keys << @variable_counter if @collection
+      keys << @variable if @has_object || @collection
+      if @collection
+        keys << @variable_counter
+        keys << @variable_iteration
+      end
       keys
     end
 
@@ -477,8 +522,11 @@ module ActionView
         raise_invalid_identifier(path) unless base =~ /\A_?([a-z]\w*)(\.\w+)*\z/
         $1.to_sym
       end
-      variable_counter = :"#{variable}_counter" if @collection
-      [variable, variable_counter]
+      if @collection
+        variable_counter = :"#{variable}_counter"
+        variable_iteration = :"#{variable}_iteration"
+      end
+      [variable, variable_counter, variable_iteration]
     end
 
     IDENTIFIER_ERROR_MESSAGE = "The partial name (%s) is not a valid Ruby identifier; " +
