@@ -7,8 +7,8 @@ module ActiveModel
   module VERSION
     MAJOR = 4
     MINOR = 2
-    TINY  = 7
-    PRE   = "1"
+    TINY  = 10
+    PRE   = nil
 
     STRING = [MAJOR, MINOR, TINY, PRE].compact.join(".")
   end
