@@ -26,12 +26,15 @@ require 'models/member'
 require 'models/membership'
 require 'models/club'
 require 'models/organization'
+require 'models/user'
+require 'models/user_business'
+require 'models/business'
 
 class HasManyThroughAssociationsTest < ActiveRecord::TestCase
   fixtures :posts, :readers, :people, :comments, :authors, :categories, :taggings, :tags,
            :owners, :pets, :toys, :jobs, :references, :companies, :members, :author_addresses,
            :subscribers, :books, :subscriptions, :developers, :categorizations, :essays,
-           :categories_posts, :clubs, :memberships, :organizations
+           :categories_posts, :clubs, :memberships, :organizations, :users, :businesses
 
   # Dummies to force column loads so query counts are clean.
   def setup
@@ -1201,4 +1204,23 @@ class HasManyThroughAssociationsTest < ActiveRecord::TestCase
   ensure
     TenantMembership.current_member = nil
   end
+
+  def test_singular_collection_ids_are_set_correctly
+    @user = users(:one)
+    assert @user.businesses.empty?
+    @user.business_ids = [businesses(:walmart).uuid]
+    assert @user.businesses.include?(businesses(:walmart))
+  end
+
+  def test_singular_collection_ids_are_added_correctly
+    user     = users(:one)
+    business = businesses(:walmart)
+    assert user.businesses.empty?
+    user.business_ids = [business.uuid]
+
+    assert_equal user.business_ids, [business.uuid]
+    assert_nothing_raised(ActiveRecord::RecordNotFound) do
+      user.business_ids += [business.uuid]
+    end
+  end
 end
