@@ -28,14 +28,33 @@ class AttributeMethodsTest < ActiveRecord::TestCase
     ActiveRecord::Base.send(:attribute_method_matchers).concat(@old_matchers)
   end
 
-  def test_attribute_for_inspect
+  def test_attribute_for_inspect_string
     t = topics(:first)
     t.title = "The First Topic Now Has A Title With\nNewlines And More Than 50 Characters"
 
-    assert_equal %("#{t.written_on.to_s(:db)}"), t.attribute_for_inspect(:written_on)
     assert_equal '"The First Topic Now Has A Title With\nNewlines And ..."', t.attribute_for_inspect(:title)
   end
 
+  def test_attribute_for_inspect_date
+    t = topics(:first)
+
+    assert_equal %("#{t.written_on.to_s(:db)}"), t.attribute_for_inspect(:written_on)
+  end
+
+  def test_attribute_for_inspect_array
+    t = topics(:first)
+    t.content = [Object.new]
+
+    assert_match %r(\[#<Object:0x[0-9a-f]+>\]), t.attribute_for_inspect(:content)
+  end
+
+  def test_attribute_for_inspect_long_array
+    t = topics(:first)
+    t.content = (1..11).to_a
+
+    assert_equal "[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]", t.attribute_for_inspect(:content)
+  end
+
   def test_attribute_present
     t = Topic.new
     t.title = "hello there!"
