@@ -96,6 +96,14 @@ module ActiveRecord
         assert ActiveRecord::Base.connection_handler.active_connections?
       end
 
+      def test_connections_closed_if_exception_and_explicitly_not_test
+        @env['rack.test'] = false
+        app       = Class.new(App) { def call(env); raise NotImplementedError; end }.new
+        explosive = ConnectionManagement.new(app)
+        assert_raises(NotImplementedError) { explosive.call(@env) }
+        assert !ActiveRecord::Base.connection_handler.active_connections?
+      end
+
       test "doesn't clear active connections when running in a test case" do
         @env['rack.test'] = true
         @management.call(@env)
