@@ -46,9 +46,8 @@ def in_memory_db?
 end
 
 def mysql_56?
-  current_adapter?(:MysqlAdapter, :Mysql2Adapter) &&
-    ActiveRecord::Base.connection.send(:version) >= '5.6.0' &&
-    ActiveRecord::Base.connection.send(:version) < '5.7.0'
+  current_adapter?(:Mysql2Adapter) &&
+    ActiveRecord::Base.connection.send(:version) >= '5.6.0'
 end
 
 def mysql_enforcing_gtid_consistency?
