@@ -18,3 +18,18 @@ class SelectedMembership < Membership
     select("'1' as foo")
   end
 end
+
+class TenantMembership < Membership
+  cattr_accessor :current_member
+
+  belongs_to :member
+  belongs_to :club
+
+  default_scope -> {
+    if current_member
+      where(member: current_member)
+    else
+      all
+    end
+  }
+end
