@@ -14,6 +14,7 @@ module ActiveSupport
 
       DIR_FORMATTER = "%03X"
       FILENAME_MAX_SIZE = 228 # max filename size on file system is 255, minus room for timestamp and random characters appended by Tempfile (used by atomic write)
+      FILEPATH_MAX_SIZE = 900 # max is 1024, plus some room
       EXCLUDED_DIRS = ['.', '..'].freeze
 
       def initialize(cache_path, options = nil)
@@ -117,6 +118,10 @@ module ActiveSupport
 
         # Translate a key into a file path.
         def key_file_path(key)
+          if key.size > FILEPATH_MAX_SIZE
+            key = Digest::MD5.hexdigest(key)
+          end
+
           fname = URI.encode_www_form_component(key)
           hash = Zlib.adler32(fname)
           hash, dir_1 = hash.divmod(0x1000)
