@@ -1,28 +1,6 @@
 require 'abstract_unit'
-require 'test/unit'
 
 module CallbacksTest
-  class Phone
-    include ActiveSupport::Callbacks
-    define_callbacks :save, :rescuable => true
-
-    set_callback :save, :before, :before_save1
-    set_callback :save, :after, :after_save1
-
-    def before_save1; self.history << :before; end
-    def after_save1; self.history << :after; end
-
-    def save
-      run_callbacks :save do
-        raise 'boom'
-      end
-    end
-
-    def history
-      @history ||= []
-    end
-  end
-
   class Record
     include ActiveSupport::Callbacks
 
@@ -67,6 +45,16 @@ module CallbacksTest
     end
   end
 
+  class CallbackClass
+    def self.before(model)
+      model.history << [:before_save, :class]
+    end
+    
+    def self.after(model)
+      model.history << [:after_save, :class]
+    end
+  end
+
   class Person < Record
     [:before_save, :after_save].each do |callback_method|
       callback_method_sym = callback_method.to_sym
@@ -74,6 +62,7 @@ module CallbacksTest
       send(callback_method, callback_string(callback_method_sym))
       send(callback_method, callback_proc(callback_method_sym))
       send(callback_method, callback_object(callback_method_sym.to_s.gsub(/_save/, '')))
+      send(callback_method, CallbackClass)
       send(callback_method) { |model| model.history << [callback_method_sym, :block] }
     end
 
@@ -87,16 +76,20 @@ module CallbacksTest
     skip_callback :save, :after, :before_save_method, :unless => :yes
     skip_callback :save, :after, :before_save_method, :if => :no
     skip_callback :save, :before, :before_save_method, :unless => :no
+    skip_callback :save, :before, CallbackClass , :if => :yes
     def yes; true; end
     def no; false; end
   end
 
+  class PersonForProgrammaticSkipping < Person
+  end
+
   class ParentController
     include ActiveSupport::Callbacks
 
     define_callbacks :dispatch
 
-    set_callback :dispatch, :before, :log, :per_key => {:unless => proc {|c| c.action_name == :index || c.action_name == :show }}
+    set_callback :dispatch, :before, :log, :unless => proc {|c| c.action_name == :index || c.action_name == :show }
     set_callback :dispatch, :after, :log2
 
     attr_reader :action_name, :logger
@@ -113,7 +106,7 @@ module CallbacksTest
     end
 
     def dispatch
-      run_callbacks :dispatch, action_name do
+      run_callbacks :dispatch do
         @logger << "Done"
       end
       self
@@ -121,7 +114,7 @@ module CallbacksTest
   end
 
   class Child < ParentController
-    skip_callback :dispatch, :before, :log, :per_key => {:if => proc {|c| c.action_name == :update} }
+    skip_callback :dispatch, :before, :log, :if => proc {|c| c.action_name == :update}
     skip_callback :dispatch, :after, :log2
   end
 
@@ -132,10 +125,10 @@ module CallbacksTest
       super
     end
 
-    before_save Proc.new {|r| r.history << [:before_save, :starts_true, :if] }, :per_key => {:if => :starts_true}
-    before_save Proc.new {|r| r.history << [:before_save, :starts_false, :if] }, :per_key => {:if => :starts_false}
-    before_save Proc.new {|r| r.history << [:before_save, :starts_true, :unless] }, :per_key => {:unless => :starts_true}
-    before_save Proc.new {|r| r.history << [:before_save, :starts_false, :unless] }, :per_key => {:unless => :starts_false}
+    before_save Proc.new {|r| r.history << [:before_save, :starts_true, :if] }, :if => :starts_true
+    before_save Proc.new {|r| r.history << [:before_save, :starts_false, :if] }, :if => :starts_false
+    before_save Proc.new {|r| r.history << [:before_save, :starts_true, :unless] }, :unless => :starts_true
+    before_save Proc.new {|r| r.history << [:before_save, :starts_false, :unless] }, :unless => :starts_false
 
     def starts_true
       if @@starts_true
@@ -154,11 +147,11 @@ module CallbacksTest
     end
 
     def save
-      run_callbacks :save, :action
+      run_callbacks :save
     end
   end
 
-  class OneTimeCompileTest < Test::Unit::TestCase
+  class OneTimeCompileTest < ActiveSupport::TestCase
     def test_optimized_first_compile
       around = OneTimeCompile.new
       around.save
@@ -177,7 +170,7 @@ module CallbacksTest
     end
   end
 
-  class AfterSaveConditionalPersonCallbackTest < Test::Unit::TestCase
+  class AfterSaveConditionalPersonCallbackTest < ActiveSupport::TestCase
     def test_after_save_runs_in_the_reverse_order
       person = AfterSaveConditionalPerson.new
       person.save
@@ -330,7 +323,7 @@ module CallbacksTest
     define_callbacks :save
     attr_reader :stuff
 
-    set_callback :save, :before, :action, :per_key => {:if => :yes}
+    set_callback :save, :before, :action, :if => :yes
 
     def yes() true end
 
@@ -339,13 +332,12 @@ module CallbacksTest
     end
 
     def save
-      run_callbacks :save, "hyphen-ated" do
+      run_callbacks :save do
         @stuff
       end
     end
   end
 
-
   module ExtendModule
     def self.extended(base)
       base.class_eval do
@@ -394,7 +386,7 @@ module CallbacksTest
       end
   end
 
-  class AroundCallbacksTest < Test::Unit::TestCase
+  class AroundCallbacksTest < ActiveSupport::TestCase
     def test_save_around
       around = AroundPerson.new
       around.save
@@ -413,7 +405,7 @@ module CallbacksTest
     end
   end
 
-  class AroundCallbackResultTest < Test::Unit::TestCase
+  class AroundCallbackResultTest < ActiveSupport::TestCase
     def test_save_around
       around = AroundPersonResult.new
       around.save
@@ -421,7 +413,7 @@ module CallbacksTest
     end
   end
 
-  class SkipCallbacksTest < Test::Unit::TestCase
+  class SkipCallbacksTest < ActiveSupport::TestCase
     def test_skip_person
       person = PersonSkipper.new
       assert_equal [], person.history
@@ -432,6 +424,26 @@ module CallbacksTest
         [:before_save, :object],
         [:before_save, :block],
         [:after_save, :block],
+        [:after_save, :class],
+        [:after_save, :object],
+        [:after_save, :proc],
+        [:after_save, :string],
+        [:after_save, :symbol]
+      ], person.history
+    end
+
+    def test_skip_person_programmatically
+      PersonForProgrammaticSkipping._save_callbacks.each do |save_callback|
+        if "before" == save_callback.kind.to_s
+          PersonForProgrammaticSkipping.skip_callback("save", save_callback.kind, save_callback.filter)
+        end
+      end
+      person = PersonForProgrammaticSkipping.new
+      assert_equal [], person.history
+      person.save
+      assert_equal [
+        [:after_save, :block],
+        [:after_save, :class],
         [:after_save, :object],
         [:after_save, :proc],
         [:after_save, :string],
@@ -440,14 +452,7 @@ module CallbacksTest
     end
   end
 
-  class CallbacksTest < Test::Unit::TestCase
-    def test_save_phone
-      phone = Phone.new
-      assert_raise RuntimeError do
-        phone.save
-      end
-      assert_equal [:before, :after], phone.history
-    end
+  class CallbacksTest < ActiveSupport::TestCase
 
     def test_save_person
       person = Person.new
@@ -458,8 +463,10 @@ module CallbacksTest
         [:before_save, :string],
         [:before_save, :proc],
         [:before_save, :object],
+        [:before_save, :class],
         [:before_save, :block],
         [:after_save, :block],
+        [:after_save, :class],
         [:after_save, :object],
         [:after_save, :proc],
         [:after_save, :string],
@@ -468,7 +475,7 @@ module CallbacksTest
     end
   end
 
-  class ConditionalCallbackTest < Test::Unit::TestCase
+  class ConditionalCallbackTest < ActiveSupport::TestCase
     def test_save_conditional_person
       person = ConditionalPerson.new
       person.save
@@ -486,7 +493,7 @@ module CallbacksTest
 
 
 
-  class ResetCallbackTest < Test::Unit::TestCase
+  class ResetCallbackTest < ActiveSupport::TestCase
     def test_save_conditional_person
       person = CleanPerson.new
       person.save
@@ -497,7 +504,7 @@ module CallbacksTest
   class CallbackTerminator
     include ActiveSupport::Callbacks
 
-    define_callbacks :save, :terminator => "result == :halt"
+    define_callbacks :save, :terminator => ->(_,result) { result == :halt }
 
     set_callback :save, :before, :first
     set_callback :save, :before, :second
@@ -616,7 +623,46 @@ module CallbacksTest
     end
   end
 
-  class UsingObjectTest < Test::Unit::TestCase
+  class OneTwoThreeSave
+    include ActiveSupport::Callbacks
+
+    define_callbacks :save
+
+    attr_accessor :record
+
+    def initialize
+      @record = []
+    end
+
+    def save
+      run_callbacks :save do
+        @record << "yielded"
+      end
+    end
+
+    def first
+      @record << "one"
+    end
+
+    def second
+      @record << "two"
+    end
+
+    def third
+      @record << "three"
+    end
+  end
+
+  class DuplicatingCallbacks < OneTwoThreeSave
+    set_callback :save, :before, :first, :second
+    set_callback :save, :before, :first, :third
+  end
+
+  class DuplicatingCallbacksInSameCall < OneTwoThreeSave
+    set_callback :save, :before, :first, :second, :first, :third
+  end
+
+  class UsingObjectTest < ActiveSupport::TestCase
     def test_before_object
       u = UsingObjectBefore.new
       u.save
@@ -641,7 +687,7 @@ module CallbacksTest
     end
   end
 
-  class CallbackTerminatorTest < Test::Unit::TestCase
+  class CallbackTerminatorTest < ActiveSupport::TestCase
     def test_termination
       terminator = CallbackTerminator.new
       terminator.save
@@ -651,7 +697,7 @@ module CallbacksTest
     def test_termination_invokes_hook
       terminator = CallbackTerminator.new
       terminator.save
-      assert_equal ":second", terminator.halted
+      assert_equal :second, terminator.halted
     end
 
     def test_block_never_called_if_terminated
@@ -661,7 +707,7 @@ module CallbacksTest
     end
   end
 
-  class HyphenatedKeyTest < Test::Unit::TestCase
+  class HyphenatedKeyTest < ActiveSupport::TestCase
     def test_save
       obj = HyphenatedCallbacks.new
       obj.save
@@ -674,7 +720,7 @@ module CallbacksTest
     skip_callback :save, :before, :before_save_method, :if => lambda {self.age > 21}
   end
 
-  class WriterCallbacksTest < Test::Unit::TestCase
+  class WriterCallbacksTest < ActiveSupport::TestCase
     def test_skip_writer
       writer = WriterSkipper.new
       writer.age = 18
@@ -685,8 +731,10 @@ module CallbacksTest
         [:before_save, :string],
         [:before_save, :proc],
         [:before_save, :object],
+        [:before_save, :class],
         [:before_save, :block],
         [:after_save, :block],
+        [:after_save, :class],
         [:after_save, :object],
         [:after_save, :proc],
         [:after_save, :string],
@@ -695,7 +743,7 @@ module CallbacksTest
     end
   end
 
-  class ExtendCallbacksTest < Test::Unit::TestCase
+  class ExtendCallbacksTest < ActiveSupport::TestCase
     def test_save
       model = ExtendCallbacks.new.extend ExtendModule
       model.save
@@ -703,4 +751,253 @@ module CallbacksTest
     end
   end
 
+  class ExcludingDuplicatesCallbackTest < ActiveSupport::TestCase
+    def test_excludes_duplicates_in_separate_calls
+      model = DuplicatingCallbacks.new
+      model.save
+      assert_equal ["two", "one", "three", "yielded"], model.record
+    end
+
+    def test_excludes_duplicates_in_one_call
+      model = DuplicatingCallbacksInSameCall.new
+      model.save
+      assert_equal ["two", "one", "three", "yielded"], model.record
+    end
+  end
+
+  class CallbackProcTest < ActiveSupport::TestCase
+    def build_class(callback)
+      Class.new {
+        include ActiveSupport::Callbacks
+        define_callbacks :foo
+        set_callback :foo, :before, callback
+        def run; run_callbacks :foo; end
+      }
+    end
+
+    def test_proc_arity_0
+      calls = []
+      klass = build_class(->() { calls << :foo })
+      klass.new.run
+      assert_equal [:foo], calls
+    end
+
+    def test_proc_arity_1
+      calls = []
+      klass = build_class(->(o) { calls << o })
+      instance = klass.new
+      instance.run
+      assert_equal [instance], calls
+    end
+
+    def test_proc_arity_2
+      assert_raises(ArgumentError) do
+        klass = build_class(->(x,y) { })
+        klass.new.run
+      end
+    end
+
+    def test_proc_negative_called_with_empty_list
+      calls = []
+      klass = build_class(->(*args) { calls << args })
+      klass.new.run
+      assert_equal [[]], calls
+    end
+  end
+
+  class ConditionalTests < ActiveSupport::TestCase
+    def build_class(callback)
+      Class.new {
+        include ActiveSupport::Callbacks
+        define_callbacks :foo
+        set_callback :foo, :before, :foo, :if => callback
+        def foo; end
+        def run; run_callbacks :foo; end
+      }
+    end
+
+    # FIXME: do we really want to support classes as conditionals?  There were
+    # no tests for it previous to this.
+    def test_class_conditional_with_scope
+      z = []
+      callback = Class.new {
+        define_singleton_method(:foo) { |o| z << o }
+      }
+      klass = Class.new {
+        include ActiveSupport::Callbacks
+        define_callbacks :foo, :scope => [:name]
+        set_callback :foo, :before, :foo, :if => callback
+        def run; run_callbacks :foo; end
+        private
+        def foo; end
+      }
+      object = klass.new
+      object.run
+      assert_equal [object], z
+    end
+
+    # FIXME: do we really want to support classes as conditionals?  There were
+    # no tests for it previous to this.
+    def test_class
+      z = []
+      klass = build_class Class.new {
+        define_singleton_method(:before) { |o| z << o }
+      }
+      object = klass.new
+      object.run
+      assert_equal [object], z
+    end
+
+    def test_proc_negative_arity # passes an empty list if *args
+      z = []
+      object = build_class(->(*args) { z << args }).new
+      object.run
+      assert_equal [], z.flatten
+    end
+
+    def test_proc_arity0
+      z = []
+      object = build_class(->() { z << 0 }).new
+      object.run
+      assert_equal [0], z
+    end
+
+    def test_proc_arity1
+      z = []
+      object = build_class(->(x) { z << x }).new
+      object.run
+      assert_equal [object], z
+    end
+
+    def test_proc_arity2
+      assert_raises(ArgumentError) do
+        object = build_class(->(a,b) { }).new
+        object.run
+      end
+    end
+  end
+
+  class ResetCallbackTest < ActiveSupport::TestCase
+    def build_class(memo)
+      klass = Class.new {
+        include ActiveSupport::Callbacks
+        define_callbacks :foo
+        set_callback :foo, :before, :hello
+        def run; run_callbacks :foo; end
+      }
+      klass.class_eval {
+        define_method(:hello) { memo << :hi }
+      }
+      klass
+    end
+
+    def test_reset_callbacks
+      events = []
+      klass = build_class events
+      klass.new.run
+      assert_equal 1, events.length
+
+      klass.reset_callbacks :foo
+      klass.new.run
+      assert_equal 1, events.length
+    end
+
+    def test_reset_impacts_subclasses
+      events = []
+      klass = build_class events
+      subclass = Class.new(klass) { set_callback :foo, :before, :world }
+      subclass.class_eval { define_method(:world) { events << :world } }
+
+      subclass.new.run
+      assert_equal 2, events.length
+
+      klass.reset_callbacks :foo
+      subclass.new.run
+      assert_equal 3, events.length
+    end
+  end
+
+  class CallbackTypeTest < ActiveSupport::TestCase
+    def build_class(callback, n = 10)
+      Class.new {
+        include ActiveSupport::Callbacks
+        define_callbacks :foo
+        n.times { set_callback :foo, :before, callback }
+        def run; run_callbacks :foo; end
+        def self.skip(thing); skip_callback :foo, :before, thing; end
+      }
+    end
+
+    def test_add_class
+      calls = []
+      callback = Class.new {
+        define_singleton_method(:before) { |o| calls << o }
+      }
+      build_class(callback).new.run
+      assert_equal 10, calls.length
+    end
+
+    def test_add_lambda
+      calls = []
+      build_class(->(o) { calls << o }).new.run
+      assert_equal 10, calls.length
+    end
+
+    def test_add_symbol
+      calls = []
+      klass = build_class(:bar)
+      klass.class_eval { define_method(:bar) { calls << klass } }
+      klass.new.run
+      assert_equal 1, calls.length
+    end
+
+    def test_add_eval
+      calls = []
+      klass = build_class("bar")
+      klass.class_eval { define_method(:bar) { calls << klass } }
+      klass.new.run
+      assert_equal 1, calls.length
+    end
+
+    def test_skip_class # removes one at a time
+      calls = []
+      callback = Class.new {
+        define_singleton_method(:before) { |o| calls << o }
+      }
+      klass = build_class(callback)
+      9.downto(0) { |i|
+        klass.skip callback
+        klass.new.run
+        assert_equal i, calls.length
+        calls.clear
+      }
+    end
+
+    def test_skip_lambda # removes nothing
+      calls = []
+      callback = ->(o) { calls << o }
+      klass = build_class(callback)
+      10.times { klass.skip callback }
+      klass.new.run
+      assert_equal 10, calls.length
+    end
+
+    def test_skip_symbol # removes all
+      calls = []
+      klass = build_class(:bar)
+      klass.class_eval { define_method(:bar) { calls << klass } }
+      klass.skip :bar
+      klass.new.run
+      assert_equal 0, calls.length
+    end
+
+    def test_skip_eval # removes nothing
+      calls = []
+      klass = build_class("bar")
+      klass.class_eval { define_method(:bar) { calls << klass } }
+      klass.skip "bar"
+      klass.new.run
+      assert_equal 1, calls.length
+    end
+  end
 end
