@@ -37,11 +37,14 @@ class InTest < ActiveSupport::TestCase
   end
   class C < B
   end
+  class D
+  end
 
   def test_in_module
     assert A.in?(B)
     assert A.in?(C)
     assert !A.in?(A)
+    assert !A.in?(D)
   end
   
   def test_no_method_catching
@@ -51,5 +54,6 @@ class InTest < ActiveSupport::TestCase
   def test_presence_in
     assert_equal "stuff", "stuff".presence_in(%w( lots of stuff ))
     assert_nil "stuff".presence_in(%w( lots of crap ))
+    assert_raise(ArgumentError) { 1.presence_in(1) }
   end
 end
