@@ -1,12 +1,9 @@
 # encoding: utf-8
 
 module MultibyteTestHelpers
-  UNICODE_STRING = 'こにちわ'
-  ASCII_STRING = 'ohayo'
-  BYTE_STRING = "\270\236\010\210\245"
-  if BYTE_STRING.respond_to?(:force_encoding)
-    BYTE_STRING.force_encoding("ASCII-8BIT")
-  end
+  UNICODE_STRING = 'こにちわ'.freeze
+  ASCII_STRING = 'ohayo'.freeze
+  BYTE_STRING = "\270\236\010\210\245".force_encoding("ASCII-8BIT").freeze
 
   def chars(str)
     ActiveSupport::Multibyte::Chars.new(str)
