@@ -6,10 +6,17 @@ module ApplicationTests
 
     def setup
       build_app
+
+      create_gemfile
+      update_boot_file_to_use_bundler
+      @old_gemfile_env = ENV["BUNDLE_GEMFILE"]
+      ENV["BUNDLE_GEMFILE"] = app_path + "/Gemfile"
     end
 
     def teardown
       teardown_app
+
+      ENV["BUNDLE_GEMFILE"] = @old_gemfile_env
     end
 
     def test_bin_setup
@@ -37,7 +44,7 @@ module ApplicationTests
       Dir.chdir(app_path) do
         app_file 'db/schema.rb', ""
 
-        output = `bin/setup 2>&1`
+        output = `bin/setup 2>&1`.gsub(/.* warning: .*\n/, "")
         assert_equal(<<-OUTPUT, output)
 == Installing dependencies ==
 The Gemfile's dependencies are satisfied
@@ -50,5 +57,16 @@ The Gemfile's dependencies are satisfied
         OUTPUT
       end
     end
+
+    private
+      def create_gemfile
+        app_file("Gemfile", "source 'https://rubygems.org'")
+        app_file("Gemfile", "gem 'rails', path: '#{RAILS_FRAMEWORK_ROOT}'", "a")
+        app_file("Gemfile", "gem 'sqlite3'", "a")
+      end
+
+      def update_boot_file_to_use_bundler
+        app_file("config/boot.rb", "require 'bundler/setup'")
+      end
   end
 end
