@@ -41,7 +41,7 @@ module ApplicationTests
     def setup
       build_app
       boot_rails
-      FileUtils.rm_rf("#{app_path}/config/environments")
+      supress_default_config
     end
 
     def teardown
@@ -49,6 +49,15 @@ module ApplicationTests
       FileUtils.rm_rf(new_app) if File.directory?(new_app)
     end
 
+    def supress_default_config
+      FileUtils.mv("#{app_path}/config/environments", "#{app_path}/config/__environments__")
+    end
+
+    def restore_default_config
+      FileUtils.rm_rf("#{app_path}/config/environments")
+      FileUtils.mv("#{app_path}/config/__environments__", "#{app_path}/config/environments")
+    end
+
     test "Rails.env does not set the RAILS_ENV environment variable which would leak out into rake tasks" do
       require "rails"
 
@@ -59,6 +68,20 @@ module ApplicationTests
       end
     end
 
+    test "lib dir is on LOAD_PATH during config" do
+      app_file 'lib/my_logger.rb', <<-RUBY
+        require "logger"
+        class MyLogger < ::Logger
+        end
+      RUBY
+      add_to_top_of_config <<-RUBY
+        require 'my_logger'
+        config.logger = MyLogger.new STDOUT
+      RUBY
+      require "#{app_path}/config/environment"
+      assert_equal 'MyLogger', Rails.application.config.logger.class.name
+    end
+
     test "a renders exception on pending migration" do
       add_to_config <<-RUBY
         config.active_record.migration_error    = :page_load
@@ -66,13 +89,25 @@ module ApplicationTests
         config.action_dispatch.show_exceptions  = true
       RUBY
 
-      require "#{app_path}/config/environment"
-      ActiveRecord::Migrator.stubs(:needs_migration?).returns(true)
-      ActiveRecord::NullMigration.any_instance.stubs(:mtime).returns(1)
+      app_file 'db/migrate/20140708012246_create_user.rb', <<-RUBY
+        class CreateUser < ActiveRecord::Migration
+          def change
+            create_table :users
+          end
+        end
+      RUBY
 
-      get "/foo"
-      assert_equal 500, last_response.status
-      assert_match "ActiveRecord::PendingMigrationError", last_response.body
+      require "#{app_path}/config/environment"
+
+      ActiveRecord::Migrator.migrations_paths = ["#{app_path}/db/migrate"]
+
+      begin
+        get "/foo"
+        assert_equal 500, last_response.status
+        assert_match "ActiveRecord::PendingMigrationError", last_response.body
+      ensure
+        ActiveRecord::Migrator.migrations_paths = nil
+      end
     end
 
     test "Rails.groups returns available groups" do
@@ -106,7 +141,7 @@ module ApplicationTests
     test "Rails::Application responds to paths" do
       require "#{app_path}/config/environment"
       assert_respond_to AppTemplate::Application, :paths
-      assert_equal AppTemplate::Application.paths["app/views"].expanded, ["#{app_path}/app/views"]
+      assert_equal ["#{app_path}/app/views"], AppTemplate::Application.paths["app/views"].expanded
     end
 
     test "the application root is set correctly" do
@@ -170,7 +205,7 @@ module ApplicationTests
 
     test "application is always added to eager_load namespaces" do
       require "#{app_path}/config/application"
-      assert Rails.application, Rails.application.config.eager_load_namespaces
+      assert_includes Rails.application.config.eager_load_namespaces, AppTemplate::Application
     end
 
     test "the application can be eager loaded even when there are no frameworks" do
@@ -254,6 +289,53 @@ module ApplicationTests
       assert_equal Pathname.new(app_path).join("somewhere"), Rails.public_path
     end
 
+    test "In production mode, config.serve_static_files is off by default" do
+      restore_default_config
+
+      with_rails_env "production" do
+        require "#{app_path}/config/environment"
+        assert_not app.config.serve_static_files
+      end
+    end
+
+    test "In production mode, config.serve_static_files is enabled when RAILS_SERVE_STATIC_FILES is set" do
+      restore_default_config
+
+      with_rails_env "production" do
+        switch_env "RAILS_SERVE_STATIC_FILES", "1" do
+          require "#{app_path}/config/environment"
+          assert app.config.serve_static_files
+        end
+      end
+    end
+
+    test "In production mode, config.serve_static_files is disabled when RAILS_SERVE_STATIC_FILES is blank" do
+      restore_default_config
+
+      with_rails_env "production" do
+        switch_env "RAILS_SERVE_STATIC_FILES", " " do
+          require "#{app_path}/config/environment"
+          assert_not app.config.serve_static_files
+        end
+      end
+    end
+
+    test "config.serve_static_assets is deprecated" do
+      require "#{app_path}/config/application"
+
+      assert_deprecated(/serve_static_assets/) do
+        app.config.serve_static_assets = false
+      end
+
+      assert_not app.config.serve_static_files
+      assert_deprecated(/serve_static_assets/) { assert_not app.config.serve_static_assets }
+
+      app.config.serve_static_files = true
+
+      assert app.config.serve_static_files
+      assert_deprecated(/serve_static_assets/) { assert app.config.serve_static_assets }
+    end
+
     test "Use key_generator when secret_key_base is set" do
       make_basic_app do |application|
         application.secrets.secret_key_base = 'b3c631c314c0bbca50c1b2843150fe33'
@@ -510,12 +592,14 @@ module ApplicationTests
     test "default method for update can be changed" do
       app_file 'app/models/post.rb', <<-RUBY
       class Post
-        extend ActiveModel::Naming
+        include ActiveModel::Model
         def to_key; [1]; end
         def persisted?; true; end
       end
       RUBY
 
+      token = "cf50faa3fe97702ca1ae"
+
       app_file 'app/controllers/posts_controller.rb', <<-RUBY
       class PostsController < ApplicationController
         def show
@@ -525,6 +609,10 @@ module ApplicationTests
         def update
           render text: "update"
         end
+
+        private
+
+        def form_authenticity_token; token; end # stub the authenticy token
       end
       RUBY
 
@@ -536,8 +624,6 @@ module ApplicationTests
 
       require "#{app_path}/config/environment"
 
-      token = "cf50faa3fe97702ca1ae"
-      PostsController.any_instance.stubs(:form_authenticity_token).returns(token)
       params = {authenticity_token: token}
 
       get "/posts/1"
@@ -568,7 +654,7 @@ module ApplicationTests
       end
 
       get "/"
-      assert last_response.body =~ /_xsrf_token_here/
+      assert_match "_xsrf_token_here", last_response.body
     end
 
     test "sets ActionDispatch.test_app" do
@@ -665,7 +751,7 @@ module ApplicationTests
     test "valid timezone is setup correctly" do
       add_to_config <<-RUBY
         config.root = "#{app_path}"
-          config.time_zone = "Wellington"
+        config.time_zone = "Wellington"
       RUBY
 
       require "#{app_path}/config/environment"
@@ -676,7 +762,7 @@ module ApplicationTests
     test "raises when an invalid timezone is defined in the config" do
       add_to_config <<-RUBY
         config.root = "#{app_path}"
-          config.time_zone = "That big hill over yonder hill"
+        config.time_zone = "That big hill over yonder hill"
       RUBY
 
       assert_raise(ArgumentError) do
@@ -687,7 +773,7 @@ module ApplicationTests
     test "valid beginning of week is setup correctly" do
       add_to_config <<-RUBY
         config.root = "#{app_path}"
-          config.beginning_of_week = :wednesday
+        config.beginning_of_week = :wednesday
       RUBY
 
       require "#{app_path}/config/environment"
@@ -698,7 +784,7 @@ module ApplicationTests
     test "raises when an invalid beginning of week is defined in the config" do
       add_to_config <<-RUBY
         config.root = "#{app_path}"
-          config.beginning_of_week = :invalid
+        config.beginning_of_week = :invalid
       RUBY
 
       assert_raise(ArgumentError) do
@@ -858,6 +944,44 @@ module ApplicationTests
       assert_match "We're sorry, but something went wrong", last_response.body
     end
 
+    test "config.action_controller.always_permitted_parameters are: controller, action by default" do
+      require "#{app_path}/config/environment"
+      assert_equal %w(controller action), ActionController::Parameters.always_permitted_parameters
+    end
+
+    test "config.action_controller.always_permitted_parameters = ['controller', 'action', 'format']" do
+      add_to_config <<-RUBY
+        config.action_controller.always_permitted_parameters = %w( controller action format )
+      RUBY
+      require "#{app_path}/config/environment"
+      assert_equal %w( controller action format ), ActionController::Parameters.always_permitted_parameters
+    end
+
+    test "config.action_controller.always_permitted_parameters = ['controller','action','format'] does not raise exeception" do
+      app_file 'app/controllers/posts_controller.rb', <<-RUBY
+      class PostsController < ActionController::Base
+        def create
+          render text: params.permit(post: [:title])
+        end
+      end
+      RUBY
+
+      add_to_config <<-RUBY
+        routes.prepend do
+          resources :posts
+        end
+        config.action_controller.always_permitted_parameters = %w( controller action format )
+        config.action_controller.action_on_unpermitted_parameters = :raise
+      RUBY
+
+      require "#{app_path}/config/environment"
+
+      assert_equal :raise, ActionController::Parameters.action_on_unpermitted_parameters
+
+      post "/posts", {post: {"title" =>"zomg"}, format: "json"}
+      assert_equal 200, last_response.status
+    end
+
     test "config.action_controller.action_on_unpermitted_parameters is :log by default on development" do
       ENV["RAILS_ENV"] = "development"
 
@@ -938,6 +1062,36 @@ module ApplicationTests
       end
     end
 
+    test "Blank config.log_level is not deprecated for non-production environment" do
+      with_rails_env "development" do
+        assert_not_deprecated do
+          make_basic_app do |application|
+            application.config.log_level = nil
+          end
+        end
+      end
+    end
+
+    test "Blank config.log_level is deprecated for the production environment" do
+      with_rails_env "production" do
+        assert_deprecated(/log_level/) do
+          make_basic_app do |application|
+            application.config.log_level = nil
+          end
+        end
+      end
+    end
+
+    test "Not blank config.log_level is not deprecated for the production environment" do
+      with_rails_env "production" do
+        assert_not_deprecated do
+          make_basic_app do |application|
+            application.config.log_level = :info
+          end
+        end
+      end
+    end
+
     test "config.log_level with custom logger" do
       make_basic_app do |application|
         application.config.logger = Logger.new(STDOUT)
@@ -970,80 +1124,90 @@ module ApplicationTests
       assert ActiveRecord::Base.dump_schema_after_migration
     end
 
-    test "rake_tasks block works at instance level" do
-      $ran_block = false
+    test "config.annotations wrapping SourceAnnotationExtractor::Annotation class" do
+      make_basic_app do |application|
+        application.config.annotations.register_extensions("coffee") do |tag|
+          /#\s*(#{tag}):?\s*(.*)$/
+        end
+      end
 
+      assert_not_nil SourceAnnotationExtractor::Annotation.extensions[/\.(coffee)$/]
+    end
+
+    test "rake_tasks block works at instance level" do
       app_file "config/environments/development.rb", <<-RUBY
         Rails.application.configure do
+          config.ran_block = false
+
           rake_tasks do
-            $ran_block = true
+            config.ran_block = true
           end
         end
       RUBY
 
       require "#{app_path}/config/environment"
+      assert_not Rails.configuration.ran_block
 
-      assert !$ran_block
       require 'rake'
       require 'rake/testtask'
       require 'rdoc/task'
 
       Rails.application.load_tasks
-      assert $ran_block
+      assert Rails.configuration.ran_block
     end
 
     test "generators block works at instance level" do
-      $ran_block = false
-
       app_file "config/environments/development.rb", <<-RUBY
         Rails.application.configure do
+          config.ran_block = false
+
           generators do
-            $ran_block = true
+            config.ran_block = true
           end
         end
       RUBY
 
       require "#{app_path}/config/environment"
+      assert_not Rails.configuration.ran_block
 
-      assert !$ran_block
       Rails.application.load_generators
-      assert $ran_block
+      assert Rails.configuration.ran_block
     end
 
     test "console block works at instance level" do
-      $ran_block = false
-
       app_file "config/environments/development.rb", <<-RUBY
         Rails.application.configure do
+          config.ran_block = false
+
           console do
-            $ran_block = true
+            config.ran_block = true
           end
         end
       RUBY
 
       require "#{app_path}/config/environment"
+      assert_not Rails.configuration.ran_block
 
-      assert !$ran_block
       Rails.application.load_console
-      assert $ran_block
+      assert Rails.configuration.ran_block
     end
 
     test "runner block works at instance level" do
-      $ran_block = false
-
       app_file "config/environments/development.rb", <<-RUBY
         Rails.application.configure do
+          config.ran_block = false
+
           runner do
-            $ran_block = true
+            config.ran_block = true
           end
         end
       RUBY
 
       require "#{app_path}/config/environment"
+      assert_not Rails.configuration.ran_block
 
-      assert !$ran_block
       Rails.application.load_runner
-      assert $ran_block
+      assert Rails.configuration.ran_block
     end
 
     test "loading the first existing database configuration available" do
@@ -1057,9 +1221,124 @@ module ApplicationTests
 
       require "#{app_path}/config/environment"
 
-      db_config =  Rails.application.config.database_configuration
+      assert_kind_of Hash, Rails.application.config.database_configuration
+    end
 
-      assert db_config.is_a?(Hash)
+    test 'raises with proper error message if no database configuration found' do
+      FileUtils.rm("#{app_path}/config/database.yml")
+      require "#{app_path}/config/environment"
+      err = assert_raises RuntimeError do
+        Rails.application.config.database_configuration
+      end
+      assert_match 'config/database', err.message
+    end
+
+    test 'config.action_mailer.show_previews defaults to true in development' do
+      Rails.env = "development"
+      require "#{app_path}/config/environment"
+
+      assert Rails.application.config.action_mailer.show_previews
+    end
+
+    test 'config.action_mailer.show_previews defaults to false in production' do
+      Rails.env = "production"
+      require "#{app_path}/config/environment"
+
+      assert_equal false, Rails.application.config.action_mailer.show_previews
+    end
+
+    test 'config.action_mailer.show_previews can be set in the configuration file' do
+      Rails.env = "production"
+      add_to_config <<-RUBY
+        config.action_mailer.show_previews = true
+      RUBY
+      require "#{app_path}/config/environment"
+
+      assert_equal true, Rails.application.config.action_mailer.show_previews
+    end
+
+    test "config_for loads custom configuration from yaml files" do
+      app_file 'config/custom.yml', <<-RUBY
+      development:
+        key: 'custom key'
+      RUBY
+
+      add_to_config <<-RUBY
+        config.my_custom_config = config_for('custom')
+      RUBY
+
+      require "#{app_path}/config/environment"
+
+      assert_equal 'custom key', Rails.application.config.my_custom_config['key']
+    end
+
+    test "config_for raises an exception if the file does not exist" do
+      add_to_config <<-RUBY
+        config.my_custom_config = config_for('custom')
+      RUBY
+
+      exception = assert_raises(RuntimeError) do
+        require "#{app_path}/config/environment"
+      end
+
+      assert_equal "Could not load configuration. No such file - #{app_path}/config/custom.yml", exception.message
+    end
+
+    test "config_for without the environment configured returns an empty hash" do
+      app_file 'config/custom.yml', <<-RUBY
+      test:
+        key: 'custom key'
+      RUBY
+
+      add_to_config <<-RUBY
+        config.my_custom_config = config_for('custom')
+      RUBY
+      require "#{app_path}/config/environment"
+
+      assert_equal({}, Rails.application.config.my_custom_config)
+    end
+
+    test "config_for with empty file returns an empty hash" do
+      app_file 'config/custom.yml', <<-RUBY
+      RUBY
+
+      add_to_config <<-RUBY
+        config.my_custom_config = config_for('custom')
+      RUBY
+      require "#{app_path}/config/environment"
+
+      assert_equal({}, Rails.application.config.my_custom_config)
+    end
+
+    test "config_for containing ERB tags should evaluate" do
+      app_file 'config/custom.yml', <<-RUBY
+      development:
+        key: <%= 'custom key' %>
+      RUBY
+
+      add_to_config <<-RUBY
+        config.my_custom_config = config_for('custom')
+      RUBY
+      require "#{app_path}/config/environment"
+
+      assert_equal 'custom key', Rails.application.config.my_custom_config['key']
+    end
+
+    test "config_for with syntax error show a more descritive exception" do
+      app_file 'config/custom.yml', <<-RUBY
+      development:
+        key: foo:
+      RUBY
+
+      add_to_config <<-RUBY
+        config.my_custom_config = config_for('custom')
+      RUBY
+
+      exception = assert_raises(RuntimeError) do
+        require "#{app_path}/config/environment"
+      end
+
+      assert_match 'YAML syntax error occurred while parsing', exception.message
     end
   end
 end
