@@ -168,7 +168,7 @@ module ApplicationTests
 
       secret = app.key_generator.generate_key('encrypted cookie')
       sign_secret = app.key_generator.generate_key('signed encrypted cookie')
-      encryptor = ActiveSupport::MessageEncryptor.new(secret, sign_secret)
+      encryptor = ActiveSupport::MessageEncryptor.new(secret[0, ActiveSupport::MessageEncryptor.key_len], sign_secret)
 
       get '/foo/read_raw_cookie'
       assert_equal 1, encryptor.decrypt_and_verify(last_response.body)['foo']
@@ -217,7 +217,7 @@ module ApplicationTests
 
       secret = app.key_generator.generate_key('encrypted cookie')
       sign_secret = app.key_generator.generate_key('signed encrypted cookie')
-      encryptor = ActiveSupport::MessageEncryptor.new(secret, sign_secret)
+      encryptor = ActiveSupport::MessageEncryptor.new(secret[0, ActiveSupport::MessageEncryptor.key_len], sign_secret)
 
       get '/foo/read_raw_cookie'
       assert_equal 1, encryptor.decrypt_and_verify(last_response.body)['foo']
@@ -276,7 +276,7 @@ module ApplicationTests
 
       secret = app.key_generator.generate_key('encrypted cookie')
       sign_secret = app.key_generator.generate_key('signed encrypted cookie')
-      encryptor = ActiveSupport::MessageEncryptor.new(secret, sign_secret)
+      encryptor = ActiveSupport::MessageEncryptor.new(secret[0, ActiveSupport::MessageEncryptor.key_len], sign_secret)
 
       get '/foo/read_raw_cookie'
       assert_equal 2, encryptor.decrypt_and_verify(last_response.body)['foo']
