@@ -38,6 +38,22 @@ module ApplicationTests
         end
       end
 
+      test "migration with empty version" do
+        Dir.chdir(app_path) do
+          output = `bin/rake db:migrate VERSION= 2>&1`
+          assert_match(/Empty VERSION provided/, output)
+
+          output = `bin/rake db:migrate:redo VERSION= 2>&1`
+          assert_match(/Empty VERSION provided/, output)
+
+          output = `bin/rake db:migrate:up VERSION= 2>&1`
+          assert_match(/VERSION is required/, output)
+
+          output = `bin/rake db:migrate:down VERSION= 2>&1`
+          assert_match(/VERSION is required - To go down one migration, use db:rollback/, output)
+        end
+      end
+
       test 'model and migration generator with change syntax' do
         Dir.chdir(app_path) do
           `rails generate model user username:string password:string;
@@ -59,7 +75,7 @@ module ApplicationTests
 
       test 'migration status when schema migrations table is not present' do
         output = Dir.chdir(app_path){ `rake db:migrate:status 2>&1` }
-        assert_equal "Schema migrations table does not exist yet.\n", output
+        assert_equal "Schema migrations table does not exist yet.\n", output.gsub(/.* warning: .*\n/, "")
       end
 
       test 'test migration status' do
