@@ -1,6 +1,7 @@
 require 'generators/generators_test_helper'
 require 'rails/generators/rails/app/app_generator'
 require 'env_helpers'
+require 'mocha/setup' # FIXME: stop using mocha
 
 class ActionsTest < Rails::Generators::TestCase
   include GeneratorsTestHelper
@@ -41,13 +42,13 @@ class ActionsTest < Rails::Generators::TestCase
   def test_add_source_adds_source_to_gemfile
     run_generator
     action :add_source, 'http://gems.github.com'
-    assert_file 'Gemfile', /source "http:\/\/gems\.github\.com"/
+    assert_file 'Gemfile', /source 'http:\/\/gems\.github\.com'/
   end
 
   def test_gem_should_put_gem_dependency_in_gemfile
     run_generator
     action :gem, 'will-paginate'
-    assert_file 'Gemfile', /gem "will\-paginate"/
+    assert_file 'Gemfile', /gem 'will\-paginate'/
   end
 
   def test_gem_with_version_should_include_version_in_gemfile
@@ -55,7 +56,7 @@ class ActionsTest < Rails::Generators::TestCase
 
     action :gem, 'rspec', '>=2.0.0.a5'
 
-    assert_file 'Gemfile', /gem "rspec", ">=2.0.0.a5"/
+    assert_file 'Gemfile', /gem 'rspec', '>=2.0.0.a5'/
   end
 
   def test_gem_should_insert_on_separate_lines
@@ -66,8 +67,8 @@ class ActionsTest < Rails::Generators::TestCase
     action :gem, 'rspec'
     action :gem, 'rspec-rails'
 
-    assert_file 'Gemfile', /^gem "rspec"$/
-    assert_file 'Gemfile', /^gem "rspec-rails"$/
+    assert_file 'Gemfile', /^gem 'rspec'$/
+    assert_file 'Gemfile', /^gem 'rspec-rails'$/
   end
 
   def test_gem_should_include_options
@@ -75,7 +76,25 @@ class ActionsTest < Rails::Generators::TestCase
 
     action :gem, 'rspec', github: 'dchelimsky/rspec', tag: '1.2.9.rc1'
 
-    assert_file 'Gemfile', /gem "rspec", github: "dchelimsky\/rspec", tag: "1\.2\.9\.rc1"/
+    assert_file 'Gemfile', /gem 'rspec', github: 'dchelimsky\/rspec', tag: '1\.2\.9\.rc1'/
+  end
+
+  def test_gem_with_non_string_options
+    run_generator
+
+    action :gem, 'rspec', require: false
+    action :gem, 'rspec-rails', group: [:development, :test]
+
+    assert_file 'Gemfile', /^gem 'rspec', require: false$/
+    assert_file 'Gemfile', /^gem 'rspec-rails', group: \[:development, :test\]$/
+  end
+
+  def test_gem_falls_back_to_inspect_if_string_contains_single_quote
+    run_generator
+
+    action :gem, 'rspec', ">=2.0'0"
+
+    assert_file 'Gemfile', /^gem 'rspec', ">=2\.0'0"$/
   end
 
   def test_gem_group_should_wrap_gems_in_a_group
@@ -89,7 +108,7 @@ class ActionsTest < Rails::Generators::TestCase
       gem 'fakeweb'
     end
 
-    assert_file 'Gemfile', /\ngroup :development, :test do\n  gem "rspec-rails"\nend\n\ngroup :test do\n  gem "fakeweb"\nend/
+    assert_file 'Gemfile', /\ngroup :development, :test do\n  gem 'rspec-rails'\nend\n\ngroup :test do\n  gem 'fakeweb'\nend/
   end
 
   def test_environment_should_include_data_in_environment_initializer_block
@@ -258,7 +277,7 @@ F
   protected
 
     def action(*args, &block)
-      silence(:stdout){ generator.send(*args, &block) }
+      capture(:stdout){ generator.send(*args, &block) }
     end
 
 end
